//
//  common.cpp
//  Common
//
//  Created by 熊田 聖也 on 12/05/11.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#include "common.h"
#include <boost/format.hpp>
#include <boost/range/algorithm.hpp>

#include <shogun/features/SimpleFeatures.h>

namespace fs = boost::filesystem;

void common::check_image(const cv::Mat& src)
{
        if ( src.empty() ) {
                throw std::runtime_error("unable to load file");
        }

        if ( src.type() != CV_8UC3 ) {
                throw std::runtime_error("type is not CV_8UC3");
        }
}

const std::string common::get_output_fullpath(
        const std::string&      input_fullpath_name, 
        const std::string&      output_dirpath_name
)
{
        fs::path input_fullpath(input_fullpath_name);
        fs::path output_dirpath(output_dirpath_name);
        return (output_dirpath / fs::basename(input_fullpath)).string() + ".bin";
}

// test ok
void common::accumulate_cvmats(const std::string& path, cv::Mat& dst)
{
        if ( fs::is_directory(path) ) {
                Accumulator accumulator(dst);
                boost::for_each(std::make_pair(fs::directory_iterator(path), fs::directory_iterator()), accumulator);
        } else {
                std::cout << path << " is not directory\n";
        }
}

void common::convert(const cv::Mat& m, shogun::CSimpleFeatures<float64_t>* features)
{
        using namespace shogun;
        assert(m.type() == CV_32FC1);
        const int rows = m.rows;
        const int cols = m.cols;
        const int size = rows * cols;
        const float32_t* ptr = reinterpret_cast<const float32_t*>(m.data);
        float64_t* matrix = SG_MALLOC(float64_t, size);
        std::copy(ptr, ptr + size, matrix);
        features->set_feature_matrix(matrix, cols, rows); // num_features(dimension), num_vectors
}

void common::Accumulator::operator()(const boost::filesystem::path& path) const
{
        if ( path.string().find(".bin") != std::string::npos ) {
                std::cout << path << std::endl;
                cv::Mat m;
                load_mat(m, path.string());
                dst_.push_back(m);        
        }
}

common::Accumulator::Accumulator(cv::Mat& dst)
        : dst_(dst) {}
                
        