//
//  common.h
//  Common
//
//  Created by 熊田 聖也 on 12/05/10.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#ifndef Common_common_h
#define Common_common_h

#include <ostream>
#include <iostream>
#include <vector>
#include <boost/program_options.hpp>
#include "Parameters.h"
#include <boost/filesystem.hpp>
#include <string>
#include <stdexcept>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/features2d/features2d.hpp>
#include <boost/math/constants/constants.hpp>
#include "cvmat_serialization.h"
#include <shogun/lib/common.h>

namespace std
{
        // used for printing "default_values(std::vector<T>&)"
        template<typename T>
        inline std::ostream& operator<<(std::ostream& os, const std::vector<T>& values) {
                for ( std::size_t i = 0, n = values.size(); i < n; ++i ) {
                        os << values[i] << " ";
                }
                return os;
        }
}

namespace shogun
{
        template<typename T>
        class CSimpleFeatures;
}

namespace common
{
        template<typename T, int N>
        inline int size_of(T (&arg)[N]) 
        {
                return N;
        }

        template<typename T>
        inline void set_params(
                boost::program_options::variables_map&  vm, 
                const std::string&                      name, 
                T&                                      param
        )
        {
                param = vm[name].as<T>();
        }
        
        template<typename T>
        inline void set_input(
                boost::program_options::variables_map&  vm, 
                Parameters<T>&                          params
        )
        {
                namespace fs = boost::filesystem;
                const std::string input = vm["input"].as<std::string>();
                params.input_ = input;
                if ( fs::is_directory(input) ) {
                        params.is_directory_ = true;
                        std::cout << "\"input\" is a directory path." << std::endl;
                } else if ( fs::is_regular(input) ) {
                        params.is_directory_ = false;
                        std::cout << "\"input\" is a file path." << std::endl;
                } else {
                        throw std::invalid_argument("\"input\" is invalid.");
                }
        }
        
        template<typename T>
        inline void set_output_directory(
                boost::program_options::variables_map&  vm, 
                Parameters<T>&                          params
        )
        {
                namespace fs = boost::filesystem;
                const std::string output = vm["output"].as<std::string>();
                params.output_ = output;
                if ( fs::exists(params.output_) ) {
                        if ( fs::is_directory(params.output_) ) {
                                // the specified directory already exists
                        } else {
                                throw std::invalid_argument("\"output\" is an existing file.");
                        }
                        
                } else {
                        fs::create_directory(params.output_);
                }
        }
        
        const std::string get_output_fullpath(
                const std::string&      input_fullpath_name, 
                const std::string&      output_dirpath_name
        );
        
        void check_image(const cv::Mat& src);
        
        const double Pi = boost::math::constants::pi<double>();
        
        inline double radian(double degree)
        {
                return degree / 180 * Pi;
        }

        template<typename T>
        void create_keypoints(
                const cv::Mat&                  src,
                const Parameters<T>&            params,
                std::vector<cv::KeyPoint>&      keypoints
        )
        {
                const int w = src.cols;
                const int h = src.rows;
                const int interval = params.interval_;
                
                for ( int j = interval; j < h; j += interval ) {
                        for ( int i = interval; i < w; i += interval ) {
                                for ( int s = 0; s < params.scales_.size(); ++s ) {
                                        for ( int a = 0; a < params.angles_.size(); ++a ) {
                                                cv::KeyPoint keypoint(
                                                        i, // x
                                                        j, // y
                                                        params.scales_[s], // diameter
                                                        radian(params.angles_[a]), 
                                                        0, // response
                                                        3, // octave
                                                        0  // class_id
                                                );
                                                keypoints.push_back(keypoint);
                                        }
                                }
                        }
                }
        }
        
        struct Accumulator
        {
                void operator()(const boost::filesystem::path& path) const;
                Accumulator(cv::Mat& dst);
                cv::Mat&        dst_;
        };

//        struct Accumulator
//        {
//                void operator()(const boost::filesystem::path& path) const
//                {
//                        if ( path.string().find(".bin") != std::string::npos ) {
//                                std::cout << path << std::endl;
//                                cv::Mat m;
//                                load_mat(m, path.string());
//                                dst_.push_back(m);        
//                        }
//                }
//                
//                Accumulator(cv::Mat& dst)
//                        : dst_(dst) {}
//                        
//                cv::Mat&        dst_;
//        };
        
        void accumulate_cvmats(const std::string& path, cv::Mat& dst);
        
        void convert(const cv::Mat& m, shogun::CSimpleFeatures<float64_t>* features);
}
#endif
