//
//  main.cpp
//  Common
//
//  Created by 熊田 聖也 on 12/05/10.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//
#ifndef UNIT_TEST

int main(int argc, const char * argv[])
{
        return 0;
}

#else // UNIT_TEST

#define BOOST_AUTO_TEST_MAIN
#include <boost/test/unit_test.hpp>
#include <iostream>
#include <opencv2/imgproc/imgproc.hpp>
#include "cvmat_serialization.h"
#include "common.h"
#include <boost/filesystem.hpp>
#include <shogun/features/SimpleFeatures.h>

namespace fs = boost::filesystem;

namespace
{        
        void test_serialization()
        {
                cv::Mat m(3, 5, CV_64F);
                for ( int j = 0; j < m.rows; ++j ) {
                        double* p = m.ptr<double>(j);
                        for ( int i = 0; i < m.cols; ++i ) {
                                p[i] = i * j;
                        }
                }
                
                {
                        save_mat(m, "test.bin");
                }
                
                {
                        cv::Mat n;
                        load_mat(n, "test.bin");
                        for ( int j = 0; j < m.rows; ++j ) {
                                const double* p = m.ptr<double>(j);
                                const double* q = n.ptr<double>(j);
                                for ( int i = 0; i < m.cols; ++i ) {
                                        BOOST_CHECK(p[i] == q[i]);
                                }
                        }        
                }
        }
        
        void test_get_output_fullpath()
        {
                const std::string input = "/usr/local/kumada/seiya.jpg";
                const std::string output = "/var/local";
                const std::string result = common::get_output_fullpath(input, output);
                BOOST_CHECK(result == "/var/local/seiya.bin");
        }
        
        void set_values(cv::Mat& m, int i)
        {
                cv::MatIterator_<double> first = m.begin<double>();
                cv::MatIterator_<double> last = m.end<double>();
                while ( first != last ) {
                        *first = i;
                        ++first;
                }
        }
        
        void test_accumulate_cvmats()
        {
                int rows = 3;
                int cols = 2;
                int n = 3;
                const std::string path_name = "/Users/kumada/Projects/scenerecognizer/SvmTrainer/SvmTrainer/src/test";
                fs::path path(path_name);
                if ( !fs::exists(path) ) {
                        fs::create_directory(path);
                }
                cv::Mat src;
                for ( int i = 0; i < n; ++i ) {
                        cv::Mat m(rows, cols, cv::DataType<double>::type);
                        set_values(m, i);
                        src.push_back(m);
                        fs::path fullpath = path / (boost::lexical_cast<std::string>(i) + ".bin"); 
                        save_mat(m, fullpath.string());
                }
                cv::Mat dst;
                common::accumulate_cvmats(path_name, dst);
                BOOST_CHECK(dst.rows == src.rows && dst.cols == src.cols);
                for ( int j = 0; j < dst.rows; ++j ) {
                        const double* dp = dst.ptr<double>(j);
                        const double* sp = src.ptr<double>(j);
                        for ( int i = 0; i < dst.cols; ++i ) {
                                BOOST_CHECK(dp[i] == sp[i]);
                        }
                }
        }
        
        void test_convert()
        {
                shogun::init_shogun_with_defaults();
                const int rows = 3;
                const int cols = 2;
                cv::Mat m(rows, cols, cv::DataType<float>::type);
                int k = 0;
                for ( int j = 0; j < rows; ++j ) {
                        float* p = m.ptr<float>(j);
                        for ( int i = 0; i < cols; ++i ) {
                                p[i] = k;
                                ++k;
                        }
                }
                shogun::CSimpleFeatures<float64_t>* features = new shogun::CSimpleFeatures<float64_t>();
                common::convert(m, features);
                
                BOOST_CHECK(rows == features->get_num_vectors());
                BOOST_CHECK(cols == features->get_num_features());
                
                for ( int j = 0, h = m.rows; j < h; ++j ) {
                        const float32_t* p = m.ptr<float32_t>(j);
                        const shogun::SGVector<float64_t>& q = features->get_feature_vector(j);
                        for ( int i = 0, w = m.cols; i < w; ++i ) {
                                BOOST_CHECK(p[i] == q[i]);
                        }
                }
                shogun::exit_shogun();
        }

}

BOOST_AUTO_TEST_CASE(TEST_main)
{       
        test_serialization();
        test_get_output_fullpath();
        test_accumulate_cvmats();
        test_convert();
        std::cout << "TEST_main\n";        
}
#endif // UNIT_TEST
